@@ -2,6 +2,6 @@ import enum
 
 
 class Interface(enum.Enum):
-    AUTODETECT = enum.auto()
     BLOCKING = enum.auto()
     ASYNC = enum.auto()
+    AUTODETECT = enum.auto()  # DEPRECATED
