@@ -1,14 +1,16 @@
 import asyncio
 import atexit
+import contextlib
 import functools
 import inspect
+import platform
 import threading
-import time
+import typing
 import warnings
+from typing import Optional
 
+from .async_wrap import wraps_by_interface
 from .callback import Callback
-from .async_wrap import wraps_by_interface, async_compat_wraps
-from .contextlib import get_ctx_mgr_cls
 from .exceptions import UserCodeException, unwrap_coro_exception, wrap_coro_exception
 from .interface import Interface
 
@@ -20,28 +22,27 @@ _BUILTIN_ASYNC_METHODS = {
 
 _RETURN_FUTURE_KWARG = "_future"
 
-# For classes and functions
-_WRAPPED_ATTR = "_SYNCHRONICITY_WRAPPED_CLS"
-_ORIGINAL_ATTR = "_SYNCHRONICITY_ORIGINAL_CLS"
-
-# For instances
-_WRAPPED_INST_ATTR = "_SYNCHRONICITY_WRAPPED_INST"
-_ORIGINAL_INST_ATTR = "_SYNCHRONICITY_ORIGINAL_INST"
-
 # Default names for classes
 _CLASS_PREFIXES = {
-    Interface.AUTODETECT: "Auto",
     Interface.BLOCKING: "Blocking",
     Interface.ASYNC: "Async",
 }
 
 # Default names for functions
 _FUNCTION_PREFIXES = {
-    Interface.AUTODETECT: "auto_",
     Interface.BLOCKING: "blocking_",
     Interface.ASYNC: "async_",
 }
 
+TARGET_INTERFACE_ATTR = "_sync_target_interface"
+SYNCHRONIZER_ATTR = "_sync_synchronizer"
+
+
+def warn_old_modal_client():
+    warnings.warn(
+        "Using latest synchronicity with an old interface - please upgrade to latest modal-client!"
+    )
+
 
 class Synchronizer:
     """Helps you offer a blocking (synchronous) interface to asynchronous code."""
@@ -54,12 +55,27 @@ class Synchronizer:
         self._multiwrap_warning = multiwrap_warning
         self._async_leakage_warning = async_leakage_warning
         self._loop = None
+        self._loop_creation_lock = threading.Lock()
         self._thread = None
         self._stopping = None
 
-        # Prep a synchronized context manager
-        self._ctx_mgr_cls = get_ctx_mgr_cls()
-        self.create(self._ctx_mgr_cls)
+        if platform.system() == "Windows":
+            # default event loop policy on windows spits out errors when
+            # closing the event loop, so use WindowsSelectorEventLoopPolicy instead
+            # https://stackoverflow.com/questions/45600579/asyncio-event-loop-is-closed-when-getting-loop
+            asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
+        # Special attribute we use to go from wrapped <-> original
+        self._wrapped_attr = "_sync_wrapped_%d" % id(self)
+        self._original_attr = "_sync_original_%d" % id(self)
+
+        # Special attribute to mark something as non-wrappable
+        self._nowrap_attr = "_sync_nonwrap_%d" % id(self)
+
+        # Prep a synchronized context manager in case one is returned and needs translation
+        self._ctx_mgr_cls = contextlib._AsyncGeneratorContextManager
+        self.create_async(self._ctx_mgr_cls)
+        self.create_blocking(self._ctx_mgr_cls)
 
         atexit.register(self._close_loop)
 
@@ -68,15 +84,6 @@ class Synchronizer:
         "_async_leakage_warning",
     ]
 
-    def get_name(self, object, interface):
-        # TODO: make it possible to override this
-        if inspect.isclass(object):
-            return _CLASS_PREFIXES[interface] + object.__name__
-        elif inspect.isfunction(object):
-            return _FUNCTION_PREFIXES[interface] + object.__name__
-        else:
-            raise Exception("Can only compute names for classes and functions")
-
     def __getstate__(self):
         return dict([(attr, getattr(self, attr)) for attr in self._PICKLE_ATTRS])
 
@@ -85,24 +92,25 @@ class Synchronizer:
             setattr(self, attr, d[attr])
 
     def _start_loop(self):
-        if self._loop and self._loop.is_running():
-            raise Exception("Synchronicity loop already running.")
+        with self._loop_creation_lock:
+            if self._loop and self._loop.is_running():
+                return self._loop
 
-        is_ready = threading.Event()
+            is_ready = threading.Event()
 
-        def thread_inner():
-            async def loop_inner():
-                self._loop = asyncio.get_running_loop()
-                self._stopping = asyncio.Event()
-                is_ready.set()
-                await self._stopping.wait()  # wait until told to stop
+            def thread_inner():
+                async def loop_inner():
+                    self._loop = asyncio.get_running_loop()
+                    self._stopping = asyncio.Event()
+                    is_ready.set()
+                    await self._stopping.wait()  # wait until told to stop
 
-            asyncio.run(loop_inner())
+                asyncio.run(loop_inner())
 
-        self._thread = threading.Thread(target=thread_inner, daemon=True)
-        self._thread.start()
-        is_ready.wait()  # TODO: this might block for a very short time
-        return self._loop
+            self._thread = threading.Thread(target=thread_inner, daemon=True)
+            self._thread.start()
+            is_ready.wait()  # TODO: this might block for a very short time
+            return self._loop
 
     def _close_loop(self):
         if self._thread is not None:
@@ -110,6 +118,7 @@ class Synchronizer:
                 # This also serves the purpose of waking up an idle loop
                 self._loop.call_soon_threadsafe(self._stopping.set)
             self._thread.join()
+            self._thread = None
 
     def _get_loop(self, start=False):
         if self._loop is None and start:
@@ -117,18 +126,21 @@ class Synchronizer:
         return self._loop
 
     def _get_running_loop(self):
+        # TODO: delete this method
         try:
             return asyncio.get_running_loop()
         except RuntimeError:
             return
 
-    def _get_runtime_interface(self, interface):
-        """Returns one out of Interface.ASYNC or Interface.BLOCKING"""
-        if interface == Interface.AUTODETECT:
-            return Interface.ASYNC if self._get_running_loop() else Interface.BLOCKING
-        else:
-            assert interface in (Interface.ASYNC, Interface.BLOCKING)
-            return interface
+    def _is_inside_loop(self):
+        loop = self._get_loop()
+        if loop is None:
+            return False
+        if threading.current_thread() != self._thread:
+            # gevent does something bad that causes asyncio.get_running_loop() to return self._loop
+            return False
+        current_loop = self._get_running_loop()
+        return loop == current_loop
 
     def _wrap_check_async_leakage(self, coro):
         """Check if a coroutine returns another coroutine (or an async generator) and warn.
@@ -154,63 +166,65 @@ class Synchronizer:
 
         return coro_wrapped()
 
-    def _wrap_instance(self, object, interface):
+    def _wrap_instance(self, obj, interface):
         # Takes an object and creates a new proxy object for it
-        interface_instances = object.__dict__.setdefault(_WRAPPED_INST_ATTR, {})
-        if interface not in interface_instances:
-            cls_dct = object.__class__.__dict__
-            interfaces = cls_dct[_WRAPPED_ATTR]
-            interface_cls = interfaces[interface]
-            new_object = interface_cls.__new__(interface_cls)
-            interface_instances[interface] = new_object
-            # Store a reference to the original object
-            new_object.__dict__[_ORIGINAL_INST_ATTR] = object
-        return interface_instances[interface]
+        cls = obj.__class__
+        cls_dct = cls.__dict__
+        interfaces = cls_dct[self._wrapped_attr]
+        if interface not in interfaces:
+            raise RuntimeError(f"Class {cls} has not synchronized {interface}.")
+        interface_cls = interfaces[interface]
+        new_obj = interface_cls.__new__(interface_cls)
+        # Store a reference to the original object
+        new_obj.__dict__[self._original_attr] = obj
+        new_obj.__dict__[SYNCHRONIZER_ATTR] = self
+        new_obj.__dict__[TARGET_INTERFACE_ATTR] = interface
+        return new_obj
 
-    def _translate_scalar_in(self, object):
+    def _translate_scalar_in(self, obj):
         # If it's an external object, translate it to the internal type
-        if hasattr(object, "__dict__"):
-            if inspect.isclass(object):  # TODO: functions?
-                return object.__dict__.get(_ORIGINAL_ATTR, object)
+        if hasattr(obj, "__dict__"):
+            if inspect.isclass(obj):  # TODO: functions?
+                return obj.__dict__.get(self._original_attr, obj)
             else:
-                return object.__dict__.get(_ORIGINAL_INST_ATTR, object)
+                return obj.__dict__.get(self._original_attr, obj)
         else:
-            return object
+            return obj
 
-    def _translate_scalar_out(self, object, interface):
+    def _translate_scalar_out(self, obj, interface):
         # If it's an internal object, translate it to the external interface
-        if inspect.isclass(object):  # TODO: functions?
-            cls_dct = object.__dict__
-            if _WRAPPED_ATTR in cls_dct:
-                return cls_dct[_WRAPPED_ATTR][interface]
+        if inspect.isclass(obj) or isinstance(obj, typing.TypeVar):  # TODO: functions?
+            cls_dct = obj.__dict__
+            if self._wrapped_attr in cls_dct:
+                return cls_dct[self._wrapped_attr][interface]
             else:
-                return object
+                return obj
         else:
-            cls_dct = object.__class__.__dict__
-            if _WRAPPED_ATTR in cls_dct:
+            cls_dct = obj.__class__.__dict__
+            if self._wrapped_attr in cls_dct:
                 # This is an *instance* of a synchronized class, translate its type
-                return self._wrap_instance(object, interface)
+                return self._wrap(obj, interface)
             else:
-                return object
+                return obj
 
-    def _recurse_map(self, mapper, object):
-        if type(object) == list:
-            return list(self._recurse_map(mapper, item) for item in object)
-        elif type(object) == tuple:
-            return tuple(self._recurse_map(mapper, item) for item in object)
-        elif type(object) == dict:
+    def _recurse_map(self, mapper, obj):
+        if type(obj) == list:
+            return list(self._recurse_map(mapper, item) for item in obj)
+        elif type(obj) == tuple:
+            return tuple(self._recurse_map(mapper, item) for item in obj)
+        elif type(obj) == dict:
             return dict(
-                (key, self._recurse_map(mapper, item)) for key, item in object.items()
+                (key, self._recurse_map(mapper, item)) for key, item in obj.items()
             )
         else:
-            return mapper(object)
+            return mapper(obj)
 
-    def _translate_in(self, object):
-        return self._recurse_map(self._translate_scalar_in, object)
+    def _translate_in(self, obj):
+        return self._recurse_map(self._translate_scalar_in, obj)
 
-    def _translate_out(self, object, interface):
+    def _translate_out(self, obj, interface):
         return self._recurse_map(
-            lambda scalar: self._translate_scalar_out(scalar, interface), object
+            lambda scalar: self._translate_scalar_out(scalar, interface), obj
         )
 
     def _translate_coro_out(self, coro, interface):
@@ -220,9 +234,7 @@ class Synchronizer:
         return unwrap_coro()
 
     def _run_function_sync(self, coro, interface):
-        current_loop = self._get_running_loop()
-        loop = self._get_loop()
-        if loop is not None and loop == current_loop:
+        if self._is_inside_loop():
             raise Exception(
                 "Deadlock detected: calling a sync function from the synchronizer loop"
             )
@@ -246,9 +258,8 @@ class Synchronizer:
     async def _run_function_async(self, coro, interface):
         coro = wrap_coro_exception(coro)
         coro = self._wrap_check_async_leakage(coro)
-        current_loop = self._get_running_loop()
         loop = self._get_loop(start=True)
-        if loop == current_loop:
+        if self._is_inside_loop():
             value = await coro
         else:
             c_fut = asyncio.run_coroutine_threadsafe(coro, loop)
@@ -275,7 +286,7 @@ class Synchronizer:
                 value = exc
                 is_exc = True
 
-    async def _run_generator_async(self, gen, interface, unwrap_user_excs=True):
+    async def _run_generator_async(self, gen, interface):
         value, is_exc = None, False
         while True:
             try:
@@ -284,11 +295,7 @@ class Synchronizer:
                 else:
                     value = await self._run_function_async(gen.asend(value), interface)
             except UserCodeException as uc_exc:
-                if unwrap_user_excs:
-                    raise uc_exc.exc from None
-                else:
-                    # This is needed since contextlib uses this function as a helper
-                    raise uc_exc
+                raise uc_exc.exc from None
             except StopAsyncIteration:
                 break
             try:
@@ -301,21 +308,45 @@ class Synchronizer:
     def create_callback(self, f, interface):
         return Callback(self, f, interface)
 
-    def _update_wrapper(self, f_wrapped, f, name=None):
+    def _update_wrapper(
+        self, f_wrapped, f, name=None, interface=None, target_module=None
+    ):
         """Very similar to functools.update_wrapper"""
         functools.update_wrapper(f_wrapped, f)
         if name is not None:
             f_wrapped.__name__ = name
             f_wrapped.__qualname__ = name
+        if target_module is not None:
+            f_wrapped.__module__ = target_module
+        setattr(f_wrapped, SYNCHRONIZER_ATTR, self)
+        setattr(f_wrapped, TARGET_INTERFACE_ATTR, interface)
 
-    def _wrap_callable(self, f, interface, name=None, allow_futures=True, unwrap_user_excs=True):
-        if hasattr(f, _ORIGINAL_ATTR):
+    def _wrap_callable(
+        self,
+        f,
+        interface,
+        name=None,
+        allow_futures=True,
+        unwrap_user_excs=True,
+        target_module=None,
+    ):
+        if hasattr(f, self._original_attr):
             if self._multiwrap_warning:
                 warnings.warn(
                     f"Function {f} is already wrapped, but getting wrapped again"
                 )
             return f
 
+        if name is None:
+            if hasattr(self, "get_name"):
+                # super dumb backwards compatibility fix
+                warn_old_modal_client()
+                name = self.get_name(f, interface)
+            else:
+                name = _FUNCTION_PREFIXES[interface] + f.__name__
+
+        is_coroutinefunction = inspect.iscoroutinefunction(f)
+
         @wraps_by_interface(interface, f)
         def f_wrapped(*args, **kwargs):
             return_future = kwargs.pop(_RETURN_FUTURE_KWARG, False)
@@ -331,7 +362,6 @@ class Synchronizer:
             # Figure out if this is a coroutine or something
             is_coroutine = inspect.iscoroutine(res)
             is_asyncgen = inspect.isasyncgen(res)
-            runtime_interface = self._get_runtime_interface(interface)
 
             if return_future:
                 if not allow_futures:
@@ -343,13 +373,17 @@ class Synchronizer:
                 else:
                     return res
             elif is_coroutine:
-                # The run_function_* may throw UserCodeExceptions that
-                # need to be unwrapped here at the entrypoint
-                if runtime_interface == Interface.ASYNC:
+                if interface == Interface.ASYNC:
                     coro = self._run_function_async(res, interface)
-                    coro = unwrap_coro_exception(coro)
+                    if not is_coroutinefunction:
+                        # If this is a non-async function that returns a coroutine,
+                        # then this is the exit point, and we need to unwrap any
+                        # wrapped exception here. Otherwise, the exit point is
+                        # in async_wrap.py
+                        coro = unwrap_coro_exception(coro)
                     return coro
-                elif runtime_interface == Interface.BLOCKING:
+                elif interface == Interface.BLOCKING:
+                    # This is the exit point, so we need to unwrap the exception here
                     try:
                         return self._run_function_sync(res, interface)
                     except UserCodeException as uc_exc:
@@ -361,9 +395,9 @@ class Synchronizer:
             elif is_asyncgen:
                 # Note that the _run_generator_* functions handle their own
                 # unwrapping of exceptions (this happens during yielding)
-                if runtime_interface == Interface.ASYNC:
+                if interface == Interface.ASYNC:
                     return self._run_generator_async(res, interface)
-                elif runtime_interface == Interface.BLOCKING:
+                elif interface == Interface.BLOCKING:
                     return self._run_generator_sync(res, interface)
             else:
                 if inspect.isfunction(res) or isinstance(
@@ -382,16 +416,22 @@ class Synchronizer:
 
                 return self._translate_out(res, interface)
 
-        self._update_wrapper(f_wrapped, f, name)
-        setattr(f_wrapped, _ORIGINAL_ATTR, f)
+        self._update_wrapper(f_wrapped, f, name, interface, target_module=target_module)
+        setattr(f_wrapped, self._original_attr, f)
         return f_wrapped
 
-    def _wrap_proxy_method(self, method, interface, allow_futures=True):
-        method = self._wrap_callable(method, interface, allow_futures=allow_futures, unwrap_user_excs=False)
+    def _wrap_proxy_method(synchronizer_self, method, interface, allow_futures=True):
+        if getattr(method, synchronizer_self._nowrap_attr, None):
+            # This method is marked as non-wrappable
+            return method
+
+        method = synchronizer_self._wrap_callable(
+            method, interface, allow_futures=allow_futures, unwrap_user_excs=False
+        )
 
         @wraps_by_interface(interface, method)
         def proxy_method(self, *args, **kwargs):
-            instance = self.__dict__[_ORIGINAL_INST_ATTR]
+            instance = self.__dict__[synchronizer_self._original_attr]
             try:
                 return method(instance, *args, **kwargs)
             except UserCodeException as uc_exc:
@@ -400,16 +440,19 @@ class Synchronizer:
         return proxy_method
 
     def _wrap_proxy_staticmethod(self, method, interface):
-        method = self._wrap_callable(method.__func__, interface)
+        orig_function = method.__func__
+        method = self._wrap_callable(orig_function, interface)
+        self._update_wrapper(method, orig_function, interface=interface)
         return staticmethod(method)
 
     def _wrap_proxy_classmethod(self, method, interface):
         method = self._wrap_callable(method.__func__, interface)
 
         @wraps_by_interface(interface, method)
-        def proxy_classmethod(wrapped_cls, *args, **kwargs):
-            return method(wrapped_cls, *args, **kwargs)
+        def proxy_classmethod(cls, *args, **kwargs):
+            return method(cls, *args, **kwargs)
 
+        self._update_wrapper(proxy_classmethod, method, interface=interface)
         return classmethod(proxy_classmethod)
 
     def _wrap_proxy_property(self, prop, interface):
@@ -420,43 +463,45 @@ class Synchronizer:
                 kwargs[attr] = self._wrap_proxy_method(func, interface, False)
         return property(**kwargs)
 
-    def _wrap_proxy_constructor(self, cls, interface):
+    def _wrap_proxy_constructor(synchronizer_self, cls, interface):
         """Returns a custom __init__ for the subclass."""
 
-        def my_init(wrapped_self, *args, **kwargs):
+        def my_init(self, *args, **kwargs):
             # Create base instance
-            args = self._translate_in(args)
-            kwargs = self._translate_in(kwargs)
+            args = synchronizer_self._translate_in(args)
+            kwargs = synchronizer_self._translate_in(kwargs)
             instance = cls(*args, **kwargs)
 
             # Register self as the wrapped one
-            interface_instances = {interface: wrapped_self}
-            instance.__dict__[_WRAPPED_INST_ATTR] = interface_instances
+            interface_instances = {interface: self}
+            instance.__dict__[synchronizer_self._wrapped_attr] = interface_instances
 
             # Store a reference to the original object
-            wrapped_self.__dict__[_ORIGINAL_INST_ATTR] = instance
+            self.__dict__[synchronizer_self._original_attr] = instance
 
-        self._update_wrapper(my_init, cls.__init__)
+        synchronizer_self._update_wrapper(my_init, cls.__init__, interface=interface)
+        setattr(my_init, synchronizer_self._original_attr, cls.__init__)
         return my_init
 
-    def _wrap_class(self, cls, interface, name):
-        if name is None:
-            name = cls.__name__
+    def _wrap_class(self, cls, interface, name, target_module=None):
         bases = tuple(
-            self._wrap_class_or_function(base, interface) if base != object else object
+            self._wrap(base, interface, require_already_wrapped=(name is not None))
+            if base != object
+            else object
             for base in cls.__bases__
         )
-        new_dict = {_ORIGINAL_ATTR: cls}
+        new_dict = {self._original_attr: cls}
         if cls is not None:
             new_dict["__init__"] = self._wrap_proxy_constructor(cls, interface)
+
         for k, v in cls.__dict__.items():
             if k in _BUILTIN_ASYNC_METHODS:
                 k_sync = _BUILTIN_ASYNC_METHODS[k]
-                if interface in (Interface.BLOCKING, Interface.AUTODETECT):
+                if interface == Interface.BLOCKING:
                     new_dict[k_sync] = self._wrap_proxy_method(
                         v, interface, allow_futures=False
                     )
-                if interface in (Interface.ASYNC, Interface.AUTODETECT):
+                if interface == Interface.ASYNC:
                     new_dict[k] = self._wrap_proxy_method(
                         v, interface, allow_futures=False
                     )
@@ -474,68 +519,132 @@ class Synchronizer:
             elif callable(v):
                 new_dict[k] = self._wrap_proxy_method(v, interface)
 
+        if name is None:
+            if hasattr(self, "get_name"):
+                # super dumb backwards compatibility fix
+                warn_old_modal_client()
+                name = self.get_name(cls, interface)
+            else:
+                name = _CLASS_PREFIXES[interface] + cls.__name__
+
         new_cls = type.__new__(type, name, bases, new_dict)
-        new_cls.__module__ = cls.__module__
+        new_cls.__module__ = cls.__module__ if target_module is None else target_module
         new_cls.__doc__ = cls.__doc__
+        if "__annotations__" in cls.__dict__:
+            new_cls.__annotations__ = cls.__annotations__  # transfer annotations
+
+        setattr(new_cls, TARGET_INTERFACE_ATTR, interface)
+        setattr(new_cls, SYNCHRONIZER_ATTR, self)
         return new_cls
 
-    def _wrap_class_or_function(self, object, interface):
-        if _WRAPPED_ATTR not in object.__dict__:
-            setattr(object, _WRAPPED_ATTR, {})
+    def _wrap(
+        self,
+        obj,
+        interface,
+        name=None,
+        require_already_wrapped=False,
+        target_module=None,
+    ):
+        # This method works for classes, functions, and instances
+        # It wraps the object, and caches the wrapped object
 
-        interfaces = object.__dict__[_WRAPPED_ATTR]
+        # Get the list of existing interfaces
+        if self._wrapped_attr not in obj.__dict__:
+            if isinstance(obj.__dict__, dict):
+                # This works for instances
+                obj.__dict__.setdefault(self._wrapped_attr, {})
+            else:
+                # This works for classes & functions
+                setattr(obj, self._wrapped_attr, {})
+
+        # If this is already wrapped, return the existing interface
+        interfaces = obj.__dict__[self._wrapped_attr]
         if interface in interfaces:
             if self._multiwrap_warning:
                 warnings.warn(
-                    f"Object {object} is already wrapped, but getting wrapped again"
+                    f"Object {obj} is already wrapped, but getting wrapped again"
                 )
             return interfaces[interface]
 
-        name = self.get_name(object, interface)
-        if inspect.isclass(object):
-            new_object = self._wrap_class(object, interface, name)
-        elif inspect.isfunction(object):
-            new_object = self._wrap_callable(object, interface, name)
+        if require_already_wrapped:
+            # This happens if a class has a custom name but its base class doesn't
+            raise RuntimeError(
+                f"{obj} needs to be serialized explicitly with a custom name"
+            )
+
+        # Wrap object (different cases based on the type)
+        if inspect.isclass(obj):
+            new_obj = self._wrap_class(
+                obj, interface, name, target_module=target_module
+            )
+        elif inspect.isfunction(obj):
+            new_obj = self._wrap_callable(
+                obj, interface, name, target_module=target_module
+            )
+        elif isinstance(obj, typing.TypeVar):
+            new_obj = self._wrap_type_var(obj, interface, name, target_module)
+        elif self._wrapped_attr in obj.__class__.__dict__:
+            new_obj = self._wrap_instance(obj, interface)
         else:
-            raise Exception("Argument %s is not a class or a callable" % object)
-        interfaces[interface] = new_object
-        return new_object
+            raise Exception("Argument %s is not a class or a callable" % obj)
 
-    def asynccontextmanager(self, func):
-        @functools.wraps(func)
-        def helper(*args, **kwargs):
-            return self._ctx_mgr_cls(func, args, kwargs)
+        # Store the interface on the obj and return
+        interfaces[interface] = new_obj
+        return new_obj
 
-        return helper
+    def _wrap_type_var(self, obj, interface, name, target_module):
+        # TypeVar translation is needed only for type stub generation, in case the
+        # "bound" attribute refers to a translatable type.
+
+        # Creates a new identical TypeVar, marked with synchronicity's special attributes
+        # This lets type stubs "translate" the `bounds` attribute on emitted type vars
+        # if picked up from module scope and in generics using the base implementation type
+
+        # TODO(elias): Refactor - since this isn't used for live apps, move type stub generation into genstub
+        new_obj = typing.TypeVar(name, bound=obj.__bound__)  # noqa
+        new_obj.__dict__[self._original_attr] = obj
+        new_obj.__dict__[SYNCHRONIZER_ATTR] = self
+        new_obj.__dict__[TARGET_INTERFACE_ATTR] = interface
+        new_obj.__module__ = target_module
+        obj.__dict__.setdefault(self._wrapped_attr, {})
+        obj.__dict__[self._wrapped_attr][interface] = new_obj
+        return new_obj
+
+    def nowrap(self, obj):
+        setattr(obj, self._nowrap_attr, True)
+        return obj
 
     # New interface that (almost) doesn't mutate objects
 
-    def create(self, object):  # TODO: this should really be __call__ later
-        if inspect.isclass(object) or inspect.isfunction(object):
-            # This is a class/function, for which we cache the interfaces
-            interfaces = {}
-            for interface in Interface:
-                interfaces[interface] = self._wrap_class_or_function(object, interface)
-            return interfaces
-        elif _WRAPPED_ATTR in object.__class__.__dict__:
-            # TODO: this requires that the class is already synchronized
-            interfaces = {}
-            for interface in Interface:
-                interfaces[interface] = self._wrap_instance(object, interface)
+    def create_blocking(
+        self, obj, name: Optional[str] = None, target_module: Optional[str] = None
+    ):
+        wrapped = self._wrap(obj, Interface.BLOCKING, name, target_module=target_module)
+        return wrapped
+
+    def create_async(
+        self, obj, name: Optional[str] = None, target_module: Optional[str] = None
+    ):
+        wrapped = self._wrap(obj, Interface.ASYNC, name, target_module=target_module)
+        return wrapped
+
+    def is_synchronized(self, obj):
+        if inspect.isclass(obj) or inspect.isfunction(obj):
+            return hasattr(obj, self._original_attr)
         else:
-            raise Exception(
-                "Can only wrap classes, functions, and instances of synchronized classes"
-            )
-        return interfaces
+            return hasattr(obj.__class__, self._original_attr)
 
-    def is_synchronized(self, object):
-        if inspect.isclass(object) or inspect.isfunction(object):
-            return hasattr(object, _ORIGINAL_ATTR)
-        else:
-            return hasattr(object.__class__, _ORIGINAL_ATTR)
+    ### DEPRECATED
+    # Only needed because old modal clients don't pin the synchronicity version,
+    # so we need this for backwards compatibility for a short while
 
-    # Old interface that we should consider purging
+    def create(self, obj):
+        warn_old_modal_client()
+        return {
+            Interface.ASYNC: self.create_async(obj),
+            Interface.BLOCKING: self.create_blocking(obj),
+        }
 
-    def __call__(self, object):
-        interfaces = self.create(object)
-        return interfaces[Interface.AUTODETECT]
+    def __call__(self, obj):
+        warn_old_modal_client()
+        return self.create_blocking(obj)
