@@ -1,11 +1,15 @@
 import asyncio
 from synchronicity import Synchronizer
 
+
 async def run():
     try:
         while True:
             print("running")
-            await asyncio.sleep(0.1)
+            await asyncio.sleep(0.3)
+    except asyncio.CancelledError:
+        print("cancelled")
+        raise
     finally:
         print("stopping")
         await asyncio.sleep(0.1)
@@ -13,4 +17,8 @@ async def run():
 
 
 s = Synchronizer()
-s(run)()
+
+try:
+    s.create_blocking(run)()
+except KeyboardInterrupt:
+    pass
