@@ -1,5 +1,4 @@
 import inspect
-import warnings
 
 from synchronicity import Synchronizer
 
@@ -10,19 +9,19 @@ def f(x):
 
 def test_multiwrap_warning(recwarn):
     s = Synchronizer(multiwrap_warning=True)
-    f_s = s(f)
+    f_s = s.create_blocking(f)
     assert f_s(42) == 1764
     assert len(recwarn) == 0
-    f_s_s = s(f_s)
+    f_s_s = s.create_blocking(f_s)
     assert f_s_s(42) == 1764
     assert len(recwarn) == 1
 
 
 def test_multiwrap_no_warning(recwarn):
     s = Synchronizer()
-    f_s = s(f)
+    f_s = s.create_blocking(f)
     assert f_s(42) == 1764
-    f_s_s = s(f_s)
+    f_s_s = s.create_blocking(f_s)
     assert f_s_s(42) == 1764
     assert len(recwarn) == 0
 
@@ -38,6 +37,6 @@ async def returns_asyncgen():
 def test_check_double_wrapped(recwarn):
     s = Synchronizer()
     assert len(recwarn) == 0
-    ret = s(returns_asyncgen)()
+    ret = s.create_blocking(returns_asyncgen)()
     assert inspect.isasyncgen(ret)
     assert len(recwarn) == 1
